;------------------------------------------------------------------------------
; sample.asm
;
; This is a sample program that demonstrates connecting a 14-bit PIC
; microcontrollers to the uM-FPU floating point coprocessor. It displays the
; results of a simple conversion by sending output to the asynchronous port.
;
; The list, #include, and CONFIG settings should be changed if another PIC
; device with compatible features is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF 

	#include umfpu.inc				;um-FPU definitions
	extern	print_setup, print_byte, print_crlf, print_string
	extern	print_float

;-------------------- uM-FPU register definitions -----------------------------

#define	DiameterIn		4			;diameter in inches	(uM-FPU register 4)
#define	Circumference	5			;circumference		(uM-FPU register 5)
#define	Area			6			;area				(uM-FPU register 6)
#define	F2_0			7			;constant 2.0		(uM-FPU register 7)
#define	F2_54			8			;constant 2.54		(uM-FPU register 8)

;-------------------- variable definitions ------------------------------------
	udata

diameterCm		res	1				;loop counter

;-------------------- string definitions --------------------------------------
; lower byte of stringTable address must be 00
; stringTable can have up to 255 bytes of data
; all strings must be zero terminated

STRINGS	code						;store strings on page 1 (PROG2)
	global	stringTable
stringTable
	addwf	PCL,f					;computed goto for strings

startMessage
	dt	0x0D, 0x0A
	dt	0x0D, 0x0A, "Conversion Example"
	dt	0x0D, 0x0A, "------------------", 0x0D, 0x0A, 0
diameterMessage1
	dt 0x0D, 0x0A, "Diameter (cm.):      25", 0
diameterMessage2
	dt 0x0D, 0x0A, "Diameter (in.):      ", 0
circumferenceMessage
	dt 0x0D, 0x0A, "Circumference (in.): ", 0
areaMessage
	dt 0x0D, 0x0A, "Area (sq.in.):       ", 0
doneMessage
	dt 0x0D, 0x0A, 0x0A, "Done.", 0x0D, 0x0A, 0
	
;-------------------- reset and interrupt vector ------------------------------

STARTUP	code
	nop								;reset vector
  	goto    main
	nop
	nop
	goto	isr						;interrupt vector

;-------------------- interrupt service routine -------------------------------

PROG1	code
isr
	retfie							;(no interrupts used)

;-------------------- main ----------------------------------------------------

main		
 	call		print_setup			;initialize the serial port

	;--------------------------------------------------------------------------
	; The following example takes an integer value representing the diameter
	; of a circle in centimeters, converts the value to inches, and calculates
	; the circumference and area in inches and square inches.
	;--------------------------------------------------------------------------

	movlw		LOW startMessage	;display startup string
	call		print_string

	;--------------------------------------------------------------------------
	; Reset the uM-FPU.
	;--------------------------------------------------------------------------

	call		fpu_reset			;reset the uM-FPU

	;--------------------------------------------------------------------------
	; Load constants for later user.
	;--------------------------------------------------------------------------

	selectA		F2_0				;store value for 2.0
	call		load_floatByte
	movlw		.2
	call		fpu_sendByte
	call		fset

	selectA		F2_54				;store value for 2.54
	call		fwriteA
	movlw		0x40
	call		fpu_sendByte
	movlw		0x22
	call		fpu_sendByte
	movlw		0x8F
	call		fpu_sendByte
	movlw		0x5C
	call		fpu_sendByte

	;--------------------------------------------------------------------------
	; Get diameter in centimeters.  The value would typically come from a sensor
	; reading, but in this example an assumed value of 25 is used.
	;--------------------------------------------------------------------------

	movlw		.25
	movwf		diameterCm

	movlw		LOW diameterMessage1
	call		print_string		;display diameter in cm

	;--------------------------------------------------------------------------
	; DiameterIn = diameterCm / 2.54
	;--------------------------------------------------------------------------

	selectA		DiameterIn			;load integer value to R0	
	call		load_floatByte		; and convert to float
	movf		diameterCm, w
	call		fpu_sendByte
	call		fset				;store R0 to DiameterIn

	selectB		F2_54				;divide by 2.54
	call		fdiv

	movlw		LOW diameterMessage2
	call		print_string		;display diameter in inches
	call		print_float

	;--------------------------------------------------------------------------
	; Circumference: DiameterIn * pi
	;--------------------------------------------------------------------------

	selectA		Circumference		;copy DiameterIn to Circumference
	selectB		DiameterIn
	call		fset
	
	call		load_pi				;load value of pi to R0
	call		fmul				;multiply Circumference by pi

	movlw		LOW circumferenceMessage
	call		print_string		;display Circumference in inches
	call		print_float

	;--------------------------------------------------------------------------
	; Area: (DiameterIn / 2) ** 2 * pi
	;--------------------------------------------------------------------------

	selectA		Area				;copy DiameterIn to Area
	selectB		DiameterIn
	call		fset
	selectB		F2_0				;divide by 2.0
	call		fdiv

	selectB		Area				;multiply Area * Area
	call		fmul

	call		load_pi				;multiply Area by pi
	call		fmul

	movlw		LOW areaMessage
	call		print_string		;display Area in inches
	call		print_float

	;--------------------------------------------------------------------------
	; End of program.
	;--------------------------------------------------------------------------

	movlw		LOW doneMessage
	call		print_string		;display diameter in inches

done
	goto		done				;end of program

	end
