;------------------------------------------------------------------------------
; fpusw_4.asm
;
; This module provides low-level interface routines for connecting 14-bit PIC
; microcontrollers to the uM-FPU floating point coprocessor.  It implements
; the SPI interface in software (bit-bang), and assumes a clock speed of 4 MHz
; (fpusw_20.asm is also available for 20 MHz operation).
;
; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; The pin assignments for FPU_CLOCK, FPU_DATAIN and FPU_DATAOUT can be changed
; as required.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list	p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific definitions
	
	extern	regA, lastRegA, regB
	extern	delay_ms, delay

;-------------------- variable definitions ------------------------------------
		udata
dataBits		res	1				;data bits
bitCount		res	1				;bit count

#define	FPU_CLOCK		PORTB, 7	;clock (uM-FPU SCLK)
#define FPU_CLOCK_DIR	TRISB, 7	;direction bit
#define	FPU_DATAIN		PORTB, 6	;data input(uM-FPU SOUT)
#define FPU_DATAIN_DIR	TRISB, 6	;direction bit
#define	FPU_DATAOUT		PORTB, 5	;data output (uM-FPU SIN)
#define FPU_DATAOUT_DIR	TRISB, 5	;direction bit

;#define	debug_enabled 1				;debug mode is enabled

#define DATA_PERIOD		.5			;data period (usec)

PROG1	code

;-------------------- fpu_reset -----------------------------------------------
; Resets the uM-FPU.
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

		global		fpu_reset
fpu_reset
	banksel		TRISB
	bcf			FPU_CLOCK_DIR		;configure FPU_CLOCK as output
	bsf			FPU_DATAIN_DIR		;configure FPU_DATAIN as input
	bcf			FPU_DATAOUT_DIR		;configure FPU_DATAOUT as output
	banksel		PORTB

	bcf			FPU_CLOCK			;reset the uM-FPU
	bcf			FPU_DATAOUT
	bsf			FPU_CLOCK
	movlw		.30					;clock high for 150 usec
	call		delay
	bcf			FPU_CLOCK
	movlw		.4					;wait 4 msec for reset to finish
	call		delay_ms

	clrf		regA				;clear variables
	clrf		lastRegA
	clrf		regB
	return

;-------------------- fpu_sendOpcode ------------------------------------------
; Waits for ready condition.
; Sends 8-bit byte to uM-FPU.
; Delays for opcode delay.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendOpcode
fpu_sendOpcode	
	btfsc		FPU_DATAIN			;wait for ready condition
	goto		fpu_sendOpcode
	call		fpu_sendByte
	movlw		DATA_PERIOD			;provide opcode delay
	call		delay	
	return

;-------------------- fpu_sendByte --------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendByte
fpu_sendByte
	movwf		dataBits			;save the byte to send
	movlw		.8					;get number of bits to send
	movwf		bitCount

fpu_sendByte2
	bcf			FPU_DATAOUT			;set data output LOW
	rlf			dataBits, f			;get next data bit
	btfsc		STATUS, C			;if next bit is 1, set data output HIGH
	bsf			FPU_DATAOUT

	bsf			FPU_CLOCK			;pulse the clock
#ifdef debug_enabled
	nop								;extend clock if debug enabled
	nop
	nop
	nop
#endif
	bcf			FPU_CLOCK

	decfsz		bitCount, f			;loop for all bits
	goto		fpu_sendByte2

#ifdef debug_enabled				;delay 600 usec
	movlw		.120
	call		delay
#else								;ensure minimum data period
	;no additional delay required with 4 MHz clock
#endif
	return

;-------------------- fpu_readByte --------------------------------------------
; Gets 8-bit byte from uM-FPU.
;	input:	none
;	output:	W register	- 8-bit value received from uM-FPU
;			dataBits	- 8-bit value received from uM-FPU
;			Z flag 		- set according to value of W register
;------------------------------------------------------------------------------

	global		fpu_readByte
fpu_readByte
	movlw		.8					;get number of bits to receive
	movwf		bitCount

fpu_readByte2
	bcf			STATUS, C			;set carry to zero
	btfsc		FPU_DATAIN			;check data input
	bsf			STATUS, C			;if HIGH, set carry to one
	rlf			dataBits, f			;store next data bit

	bsf			FPU_CLOCK			;pulse the clock
#ifdef debug_enabled
	nop								;extend clock if debug enabled
	nop
	nop
	nop
#endif
	bcf			FPU_CLOCK

	decfsz		bitCount, f			;loop for all bits
	goto		fpu_readByte2

#ifdef debug_enabled				;delay 600 usec
	movlw		.120
	call		delay
#else								;ensure minimum data period
	;no additional delay required with 4 MHz clock
#endif

	movf		dataBits, w			;get the return byte
	return

;-------------------- fpu_traceDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_traceDelay
fpu_traceDelay
	movlw		.3					;delay for 3 msec after trace on/off
	call		delay_ms	
	return

;-------------------- fpu_readDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_readDelay
fpu_readDelay
	movlw		DATA_PERIOD			;delay 25 usec for setup before read
	call		delay	
	return

	end

