;------------------------------------------------------------------------------
; graph.asm
;
; This is a sample program that demonstrates connecting a 14-bit PIC
; microcontrollers to the uM-FPU floating point coprocessor. It displays a
; simple graph of the sine and cosine functions, by sending output to the
; asynchronous port.
;
; The list, #include, and CONFIG settings should be changed if another PIC
; device with compatible features is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _HS_OSC & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF 

	#include umfpu.inc				;um-FPU definitions
	extern	print_setup, print_byte, print_crlf, print_string
	extern	print_floatFormat

;-------------------- uM-FPU register definitions -----------------------------

#define	Angle			1			;angle			(uM-FPU register 1)
#define	Point			2			;point on graph	(uM-FPU register 2)
#define	Pi				3			;constant pi	(uM-FPU register 3)
#define	F10_0			4			;constant 10.0	(uM-FPU register 4)
#define	F20_0			5			;constant 20.0	(uM-FPU register 5)

;-------------------- variable definitions ------------------------------------
	udata

xval	res	1						;x value for graph
yval	res	1						;y value for graph
pval	res	1						;point value for graph

;-------------------- string definitions --------------------------------------
; lower byte of stringTable address must be 00
; stringTable can have up to 255 bytes of data
; all strings must be zero terminated

STRINGS	code
	global	stringTable
stringTable
	addwf	PCL,f					;computed goto for strings

startMessage
	dt	0x0D, 0x0A
	dt	0x0D, 0x0A, "Graphdemo V1.0"
	dt	0x0D, 0x0A, "--------------", 0x0D, 0x0A, 0
sinString
	dt 0x0D, 0x0A, 0x0D, 0x0A, "Graph of sine", 0
cosString
	dt 0x0D, 0x0A, 0x0D, 0x0A, "Graph of cosine", 0
axisString
	dt 0x0D, 0x0A, "-1        0         1"
	dt 0x0D, 0x0A, "---------------------", 0x0D, 0x0A, 0
doneMessage
	dt 0x0D, 0x0A, 0x0A, "Done.", 0x0D, 0x0A, 0
	
;-------------------- reset and interrupt vector ------------------------------

STARTUP	code
	nop								;reset vector
  	goto    main
	nop
	nop
	goto	isr						;interrupt vector

;-------------------- interrupt service routine -------------------------------

PROG	code
isr
	retfie							;(no interrupts used)

;-------------------- main ----------------------------------------------------

main		
 	call		print_setup			;initialize the serial port

	;--------------------------------------------------------------------------
	; The following example draws a simple graph of the sine function calculated
	; at 20 points from x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW startMessage	;display startup string
	call		print_string

	;--------------------------------------------------------------------------
	; Reset the uM-FPU.
	;--------------------------------------------------------------------------

	call		fpu_reset			;reset the uM-FPU

	;--------------------------------------------------------------------------
	; Load constants for later user.
	;--------------------------------------------------------------------------

	selectA		F10_0				;set constant 10.0
	call		load_floatByte
	movlw		.10
	call		fpu_sendByte
	call		fset

	selectA		F20_0				;set constant 20.0
	selectB		F10_0				;(F20_0 = F10_0 + F10_0)
	call		fset
	call		fadd
	
	;--------------------------------------------------------------------------
	; Draws a simple graph of the sine function calculated at 20 points from
	; x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW sinString
	call		print_string		;display diameter in cm
	movlw		LOW axisString
	call		print_string		;display diameter in cm

	clrf		xval				;clear x value

sinLoop
	selectA		Angle				;Angle = pi
	call		load_pi
	call		fset

	call		load_floatByte		;Angle = Angle * xval * 2
	movf		xval, w
	addwf		xval, w
	call		fpu_sendByte
	call		fmul

	selectB		F20_0				;Angle = Angle / 20.0
	call		fdiv

	call		sin					;Angle = sin(Angle)

	call		graph				;graph the point

	incf		xval, f				;increment the y value
	movf		xval, w				;check if done
	sublw		.20
	btfsc		STATUS, C
	goto		sinLoop				;no, then display next y value

	;--------------------------------------------------------------------------
	; Draws a simple graph of the cosine function calculated at 20 points from
	; x = 0 to x = 2pi.
	;--------------------------------------------------------------------------

	movlw		LOW cosString
	call		print_string		;display diameter in cm
	movlw		LOW axisString
	call		print_string		;display diameter in cm

	clrf		xval				;clear x value

cosLoop
	selectA		Angle				;Angle = pi
	call		load_pi
	call		fset

	call		load_floatByte		;Angle = Angle * xval * 2
	movf		xval, w
	addwf		xval, w
	call		fpu_sendByte
	call		fmul

	selectB		F20_0				;Angle = Angle / 20.0
	call		fdiv

	call		cos					;Angle = cos(Angle)

	call		graph				;graph the point

	incf		xval, f				;increment the y value
	movf		xval, w				;check if done
	sublw		.20
	btfsc		STATUS, C
	goto		cosLoop				;no, then display next y value
	
	;--------------------------------------------------------------------------
	; End of program.
	;--------------------------------------------------------------------------

	movlw		LOW doneMessage
	call		print_string		;display diameter in inches

done
	goto		done				;end of program

;-------------------- graph -----------------------------------------------
; draw simple graph of values
;--------------------------------------------------------------------------

graph
	selectA		Point				;Point = Angle * 10.0
	selectB		Angle
	call		fset
	selectB		F10_0
	call		fmul

	call		round				;Reg0 = fix(round(Point))
	call		fix

	call		fread				;read the low 8 bits of Reg0
	movf		dataByte, w
	addlw		.10					;add offset to origin
	movwf		pval

	clrf		yval				;clear the y value

graphLoop:
	movf		yval, w				;if y value = point value display asterisk
	subwf		pval, w
	btfss		STATUS, Z
	goto		graph2
	movlw		'*'
	goto		graph4

graph2
	movf		yval, w				;elseif y value = 10 display vertical bar
	sublw		.10
	btfss		STATUS, Z
	goto		graph3
	movlw		'|'
	goto		graph4

graph3
	movlw		' '					;else display blank

graph4
	call		print_byte			;display the byte

	incf		yval, f				;increment the y value
	movf		yval, w
	sublw		.20					;check if done
	btfsc		STATUS, C
	goto		graphLoop			;no, then display next y value

	movlw		' '					;display space
	call		print_byte

	selectA		Angle				;display Angle value in 8.5 format
	movlw		.85
	call		print_floatFormat

	call		print_crlf			;display CR, LF and exit
	return

	end
