;------------------------------------------------------------------------------
; serial.asm
;
; This module provides routines to output data to the serial port using the
; hardware asynchronous interface. It also supports printing strings received
; from the uM-FPU.
;
; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; The baud rate setting depends on the clock speed used (refer to the appropriate
; PIC datasheet). Sample settings are as follows:
;
;							  BRGH	  SPBRG
;							  ----	  -----					
; 		20 MHz	19,200 baud		1		64
; 		4 MHz	19,200 baud		1		12
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list	p=16f628					;list directive to define processor
	#include <p16f628.inc>				;processor specific definitions

	#include "umfpu.inc"		
	extern	opcode
	extern	stringTable

;-------------------- local variables -----------------------------------------
	udata

#define RX_DIR		TRISB, 1			;RX direction bit
#define TX_DIR		TRISB, 2			;TX direction bit

sptr	res	1							;string pointer low 8 bits
tmp		res	1							;temporary value

PROG	code

;-------------------- print_setup ---------------------------------------------
; Initialize the serial port (defaults to 19,200 baud 8,N,1
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_setup
print_setup
	banksel		TRISB
	bsf			RX_DIR					;configure RX as input
	bcf			TX_DIR					;configure TX as output
	banksel		PORTB

	movlw	.12							;set baud rate to 19,200 (assumes 20 MHz)
	banksel	SPBRG
	movwf	SPBRG
	movlw	0x26						;set TXEN, BRGH, TRMT in TXSTA
	movwf	TXSTA
	movlw	0x90						;set SPEN, CREN in RCSTA
	banksel	RCSTA
	movwf	RCSTA
	return

;-------------------- print_float ---------------------------------------------

	global	print_float, print_floatFormat
print_float
	clrw								;set format to zero
	;(fall through to print_floatFormat)

;-------------------- print_floatFormat ---------------------------------------

print_floatFormat
	movwf	opcode						;save the format value
	call	ftoa						;convert floating point to ascii
	goto	print

;-------------------- print_long ----------------------------------------------

	global	print_long, print_longFormat
print_long
	clrw								;set format to zero
	;(fall through to print_longFormat)

;-------------------- print_longFormat -----------------------------------------

print_longFormat
	movwf	opcode						;save the format value
	call	ltoa						;convert long to ascii
	;(fall through to print)
		
print
	movf	opcode, w					;send the format byte
	call	fpu_sendByte
	goto	print_fpuString				;read string and send to serial port

;-------------------- print_fpuStr --------------------------------------------
; Read zero terminated string from um_FPU and send to serial port
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_fpuString
print_fpuString
	call	readstr						;setup to read string

printFpuString2							;read zero terminated string
	call	fpu_readByte				;read the next byte
	btfsc	STATUS, Z					;check for zero terminator
	return								;yes, then exit

	call	print_byte					;no, then send the byte
	goto	printFpuString2				;get next byte

;-------------------- print_string ---------------------------------------------
; Send zero terminated string to serial port
;	input:	W register	low byte of string address
;	output:	none
;------------------------------------------------------------------------------

	global	print_string
print_string
	movwf	sptr						;save the string offset
	decf	sptr, f						;adjust for computed goto

print_string2
	movlw	HIGH stringTable			;get high byte of string address (in case
	movwf	PCLATH						; stringTable stored on different page)

	movf	sptr, w						;get low byte of string address
	call	stringTable					;get the next character

	clrf	PCLATH						;reset PCLATH to page zero (in case
										; stringTable stored on different page)
	iorlw	0
	btfsc	STATUS, Z					;check for zero
	return								;yes, then exit 
	call	print_byte					;no, then send character to serial port
	incf	sptr, f
	goto	print_string2				;get next character

;-------------------- print_hex32 ---------------------------------------------
; Send 32-bit hex value to serial port
;	input:	dataByte, dataByte+1, dataByte+2, dataByte+3
;	output:	none
;------------------------------------------------------------------------------

	global	print_hex32
print_hex32
	movf		dataByte+3, w
	call		print_hex
	movf		dataByte+2, w
	call		print_hex
	movf		dataByte+1, w
	call		print_hex
	movf		dataByte, w
	;(fall through to print_hex)

;-------------------- print_hex -----------------------------------------------
; Send 8-bit hex value to serial port
;	input:	W register	8-bit hex value
;	output:	none
;------------------------------------------------------------------------------

	global	print_hex
print_hex
	movwf	tmp							;save the original value
	swapf	tmp, w						;display upper hex digit
	call	print_hexDigit
	movf	tmp, w						;display lower hex digit			
	;(fall through to print_hexDigit)

;-------------------- print_hexDigit ------------------------------------------
; Send 4-bit hex value to serial port
;	input:	W register	lower 4 bits contain hex value
;	output:	none
;------------------------------------------------------------------------------

	global	print_hexDigit
print_hexDigit
	andlw	0x0F						;get the lower 4 bits
	addlw	6							;check for A-F digit
	btfsc	STATUS, DC
	addlw	7							;yes, then adjust
	addlw	'0'-6						;convert to ASCII character
	goto	print_byte					;send hex digit

;-------------------- print_crlf ----------------------------------------------
; Send carraige return, linefeed to serial port
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

	global	print_crlf
print_crlf
	movlw	0x0D						;send carraige return
	call	print_byte
	movlw	0x0A						;send linefeed
	;(fall through to printByte)

;-------------------- print_byte -----------------------------------------------
; Send byte to serial port
;	input:	W register	8-bit byte
;	output:	none
;------------------------------------------------------------------------------

	global	print_byte
print_byte
	btfss	PIR1, TXIF					;wait for buffer empty
	goto	print_byte
	movwf	TXREG						;send byte to serial port
	return
	end
