;------------------------------------------------------------------------------
; fpu877hw_20.asm
;
; This module provides low-level interface routines for connecting a PIC16F877
; microcontroller to the uM-FPU floating point coprocessor.  It uses the
; PIC16F877 Master Synchronous Serial Port configured for SPI mode, and assumes
; a clock speed of 20 MHz. The hardware SPI is configured using the TM2 timer
; for a clock source. (There is a 4 MHz version of this file that uses the
; oscillator divided down, and does not use the TM2 timer.)
;
; The list and #include directives should be changed if another PIC device is
; used that has compatible features.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list	p=16f877				;list directive to define processor
	#include <p16f877.inc>			;processor specific definitions
	
	extern	regA, lastRegA, regB
	extern	delay_ms, delay

;-------------------- variable definitions ------------------------------------
		udata
dataBits		res	1				;data bits

#define	FPU_CLOCK		PORTC, 3	;clock (uM-FPU SCLK)
#define FPU_CLOCK_DIR	TRISC, 3	;direction bit
#define	FPU_DATAIN		PORTC, 4	;data input(uM-FPU SOUT)
#define FPU_DATAIN_DIR	TRISC, 4	;direction bit
#define	FPU_DATAOUT		PORTC, 5	;data output (uM-FPU SIN)
#define FPU_DATAOUT_DIR	TRISC, 5	;direction bit

;#define debug_enabled 1				;debug mode is enabled (comment out to disable)

#define	TRANSMIT_TIME	.10			;time required to transmit 8 bits (usec)
#define DATA_PERIOD		.25			;data period (usec)

PROG1	code

;-------------------- fpu_reset -----------------------------------------------
; Resets the uM-FPU.
;	input:	none
;	output:	none
;------------------------------------------------------------------------------

		global		fpu_reset
fpu_reset
	banksel		TRISC
	bcf			FPU_CLOCK_DIR		;configure FPU_CLOCK as output
	bsf			FPU_DATAIN_DIR		;configure FPU_DATAIN as input
	bcf			FPU_DATAOUT_DIR		;configure FPU_DATAOUT as output
	banksel		PORTC

	bcf			FPU_CLOCK			;reset the uM-FPU
	bcf			FPU_DATAOUT
	bsf			FPU_CLOCK
	movlw		.150				;clock high for 150 usec
	call		delay
	bcf			FPU_CLOCK
	movlw		.4					;wait 4 msec for reset to finish
	call		delay_ms

	clrf		regA				;clear variables
	clrf		lastRegA
	clrf		regB

	bcf			SSPCON, SSPEN		;disable the SPI module
  	clrf		T2CON
	#ifdef debug_enabled
	  movlw		.19					;SPI clock = 125 KHz (20 MHz / 4 / 20 / 2)
	#else
	  movlw		.2					;SPI clock = 833 KHz (20 MHz / 4 / 3 / 2)
	#endif
	banksel		PR2
	movwf		PR2
	banksel		T2CON
	bsf			T2CON, TMR2ON
	movlw		B'00000011'			;CKP=0, clock idle low, clk=timer2/2
	movwf		SSPCON
	movlw		B'01000000'			;SMP=0, CKE=1, sample on rising edge
	banksel		SSPSTAT
	movwf		SSPSTAT
	banksel		SSPCON
	bsf			SSPCON, SSPEN		;enable the SPI module
	return

;-------------------- fpu_sendOpcode ------------------------------------------
; Waits for ready condition.
; Sends 8-bit byte to uM-FPU.
; Delays for opcode delay.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendOpcode
fpu_sendOpcode	
	btfsc		FPU_DATAIN			;wait for ready condition
	goto		fpu_sendOpcode
	call		fpu_sendByte
	movlw		TRANSMIT_TIME		;provide opcode delay
	call		delay	
	return

;-------------------- fpu_sendByte --------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_sendByte
fpu_sendByte
	movwf		SSPBUF				;send byte to SPI

	banksel		SSPSTAT				;wait for transmit to finish
fpu_sendByte2
	btfss		SSPSTAT, BF
	goto		fpu_sendByte2
	banksel		SSPBUF
	
  #ifdef debug_enabled				;delay 600 usec
	movlw		.200
	call		delay
	movlw		.200
	call		delay
	movlw		.200
	call		delay
  #else 							;ensure minimum data period
	movlw		DATA_PERIOD-TRANSMIT_TIME
	call		delay
  #endif

	return

;-------------------- fpu_readByte --------------------------------------------
; Gets 8-bit byte from uM-FPU.
;	input:	none
;	output:	W register	- 8-bit value received from uM-FPU
;			dataBits	- 8-bit value received from uM-FPU
;			Z flag 		- set according to value of W register
;------------------------------------------------------------------------------

	global		fpu_readByte
fpu_readByte
	clrf		SSPBUF				;send zero to SPI

	banksel		SSPSTAT				;wait for receive to finish
fpu_readByte2
	btfss		SSPSTAT, BF
	goto		fpu_readByte2
	banksel		SSPBUF
	
  #ifdef debug_enabled				;delay 600 usec
	movlw		.200
	call		delay
	movlw		.200
	call		delay
	movlw		.200
	call		delay
  #else								;ensure minimum data period
	movlw		DATA_PERIOD-TRANSMIT_TIME
	call		delay
  #endif

	movf		SSPBUF, w			;get the input byte
	movwf		dataBits
	return

;-------------------- fpu_traceDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_traceDelay
fpu_traceDelay
	movlw		.3					;delay for 3 msec after trace on/off
	call		delay_ms	
	return

;-------------------- fpu_readDelay -------------------------------------------
; Sends 8-bit byte to uM-FPU.
;	input:	W register	- 8-bit value to send to uM-FPU
;	output:	none
;------------------------------------------------------------------------------

	global		fpu_readDelay
fpu_readDelay
	movlw		DATA_PERIOD			;delay 25 usec for setup before read
	call		delay	
	return

	end
