;------------------------------------------------------------------------------
; umfpu.asm
;
; This module provides support for each uM-FPU command.
;
; Note: The code as supplied assumes an PIC16F628 processor. The list and
; #include directives should be changed if a different 14-bit microcontroller
; is used.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 September 20, 2004
; changes:
;	V1.0 - September 20, 2004
;	  - original code
;------------------------------------------------------------------------------

	list     p=16f877				;list directive to define processor
	#include <p16f877.inc>			;processor specific definitions

	extern	fpu_sendOpcode, fpu_sendByte, fpu_readByte
	extern	fpu_traceDelay, fpu_readDelay

;-------------------- global variable definitions -----------------------------

		udata
regA			res	1				;register A
regB			res	1				;register B
lastRegA		res	1				;last value of register A
opcode			res	1				;opcode value
dataByte		res	4				;data byte
		global	regA, regB, lastRegA, opcode, dataByte

PROG	code

;-------------------- floating point routines ---------------------------------
; Note: Unused routines can be commented out to reduce memory space.
;------------------------------------------------------------------------------

fwriteA
	global		fwriteA
	movf		regA, w				;write floating point value to A
	movwf		lastRegA			;update last regA value
	iorlw		0x20				;send opcode
	goto		fpu_sendOpcode		;(must be followed by 4 fpu_sendByte calls)

fwriteB
	global		fwriteB
	movf		regB, w				;write floating point value to B
	iorlw		0x30				;send opcode
	goto		fpu_sendOpcode		;(must be followed by 4 fpu_sendByte calls)
fread
	global		fread
	movlw		0x40				;get floating point value of A
	iorwf		regA, w				;add value of regA to opcode
	call		sendCommand
	goto		get_long			;store 32-bit value in dataByte to dataByte+3
fset
	global		fset
	movlw		0x50				;A = B
	goto		sendOperator
fadd
	global		fadd
	movlw		0x60				;A = A + B
	goto		sendOperator
fsub
	global		fsub
	movlw		0x70				;A = A - B
	goto		sendOperator
fmul
	global		fmul
	movlw		0x80				;A = A * B
	goto		sendOperator
fdiv
	global		fdiv
	movlw		0x90				;A = A / B
	goto		sendOperator
sqrt
	global		sqrt
	movlw		0xE0				;A = sqrt(A)
	goto		sendCommand
log
	global		log
	movlw		0xE1				;A = log(A)
	goto		sendCommand
log10
	global		log10
	movlw		0xE2				;A = log10(A)
	goto		sendCommand
exp
	global		exp
	movlw		0xE3				;A = exp(A)
	goto		sendCommand
exp10
	global		exp10
	movlw		0xE4				;A = exp10(A)
	goto		sendCommand
sin
	global		sin
	movlw		0xE5				;A = sin(A) radians
	goto		sendCommand
cos
	global		cos
	movlw		0xE6				;A = cos(A) radians
	goto		sendCommand
tan
	global		tan
	movlw		0xE7				;A = tan(A) radians
	goto		sendCommand
floor
	global		floor
	movlw		0xE8				;A = nearest integer <= A
	goto		sendCommand
ceil
	global		ceil
	movlw		0xE9				;A = nearest integer >= A
	goto		sendCommand
round
	global		round
	movlw		0xEA				;A = nearest integer to A
	goto		sendCommand
negate
	global		negate
	movlw		0xEB				;A = -A
	goto		sendCommand
abs
	global		abs
	movlw		0xEC				;A = |A|
	goto		sendCommand
inverse
	global		inverse
	movlw		0xED				;A = 1 / A
	goto		sendCommand
degrees
	global		degrees
	movlw		0xEE				;A = A / (PI / 180.0) radians to degrees
	goto		sendCommand
radians
	global		radians
	movlw		0xEF				;A = A * (PI / 180.0) degrees to radians
	goto		sendCommand
fcompare
	global		fcompare
	movlw		0xF3				;compare registers A and B (A - B)
	call		sendCommandB
	call		fpu_readDelay		;add extra read delay
	call		fpu_readDelay
	goto		get_byte		
atof
	global		atof
	clrf		regB				;clear B register
	movlw		0xF9				;convert ASCII string to float
	goto		sendCommand			;(must be followed by string write)
ftoa
	global		ftoa
	movlw		0xFA				;convert float to ASCII string
	goto		sendCommand			;(must be followed by string read)	
fstatus
	global		fstatus
	movlw		0xFD				;get floating point status of A
	call		sendCommand
	call		fpu_readDelay		;add extra read delay
	goto		get_byte
pow
	global		pow
	movlw		0xE0				;A = A to the power of B
	goto		sendCommandB
root
	global		root
	movlw		0xE1				;A = the Bth root of A
	goto		sendCommandB
min
	global		min
	movlw		0xE2				;A = minimum of A AND B
	goto		sendCommandB
max
	global		max
	movlw		0xE3				;A = maximum of A AND B
	goto		sendCommandB
fraction
	global		fraction
	clrf		regB				;clear B register
	movlw		0xE4				;load register 0 with fractional part of A
	goto		loadCommand
asin
	global		asin
	movlw		0xE5				;A = asin(A) radians
	goto		sendXcommand
acos
	global		acos
	movlw		0xE6				;A = acos(A) radians
	goto		sendXcommand
atan
	global		atan
	movlw		0xE7				;A = atan(A) radians
	goto		sendXcommand
atan2
	global		atan2
	movlw		0xE8				;A = atan(A/B)
	goto		sendCommandB

;-------------------- Long Integer Routines -----------------------------------
; Note: Unused routines can be commented out to reduce memory space.
;------------------------------------------------------------------------------

lwriteA
	global		lwriteA
	movf		regA, w				;write floating point value to A
	movwf		lastRegA			;update last regA value
	iorlw		0xA0				;send opcode
	goto		sendXcommand		;(must be followed by 4 fpu_sendByte calls)
lwriteB
	global		lwriteB
	movf		regB, w				;write floating point value to B
	iorlw		0xB0				;send opcode
	goto		sendXcommand		;(must be followed by 4 fpu_sendByte calls)
lread
	global		lread
	movlw		0xC0				;get value of A
	iorwf		regA, w				;add value of regA to opcode
	call		sendXcommand
	goto		get_long			;store 32-bit value in dataByte to dataByte+3	
lset
	global		lset
	movlw		0x50				;A = B
	goto		sendOperator
ladd
	global		ladd
	movlw		0xA0				;A = A + B (long integer)
	goto		sendOperator
lsub
	global		lsub
	movlw		0xB0				;A = A - B (long integer)
	goto		sendOperator
lmul
	global		lmul
	movlw		0xC0				;A = A * B (long integer)
	goto		sendOperator
ldiv
	global		ldiv
	movlw		0xD0				;A = A / B (signed long integer)
	goto		sendOperator
atol
	global		atol
	clrf		regB				;clear B register
	movlw		0xFB				;convert ASCII string to long
	goto		sendCommand			;(must be followed by string write)	
ltoa
	global		ltoa
	movlw		0xFC				;convert long to ASCII string
	goto		sendCommand			;(must be followed by string read)	
ludiv
	global		ludiv
	movlw		0xD0				;A = A / B (unsigned long integer)
	iorwf		regB, w				;add value of regB to opcode
	goto		sendXcommand
lcompare
	global		lcompare
	movlw		0xE9				;compare A and B (signed long integer)
	call		sendCommandB
	call		fpu_readDelay		;add extra read delay
	goto		get_byte
lucompare
	global		lucompare
	movlw		0xEA				;compare A and B (unsigned long integer)
	call		sendCommandB
	goto		get_byte
lstatus
	global		lstatus
	movlw		0xEB				;get status of A (long integer)
	call		sendXcommand
	goto		get_byte
lnegate
	global		lnegate
	movlw		0xEC				;A = -A (long integer)
	goto		sendXcommand
labs
	global		labs
	movlw		0xED				;A = |A| (long integer)
	goto		sendXcommand

;-------------------- misc routines -------------------------------------------

sync
	global		sync
	movlw		0xF0				;get sync character
	call		sendCommand
	goto		get_byte
float
	global		float
	movlw		0xF1				;A = float(A), (convert long to float)
	goto		convert
fix
	global		fix
	movlw		0xF2				;A = fix(A), (convert float to long)
	;(falls though to convert)
convert
	call		sendCommand
	clrf		regB
	movlw		0x50
	goto		sendCommand
readstr
	global		readstr
	movlw		0xF8				;setup to read string
	call		sendCommand
	goto		fpu_readDelay		;delay before read
left
	global		left
	movlw		0xEE				;left parenthesis
	goto		sendXcommand
right
	global		right
	clrf		regB				;clear B register
	movlw		0xEF				;right parenthesis
	goto		sendXcommand
ieeeMode
	global		ieeeMode
	movlw		0xF8				;IEEE 754 mode (default at reset)
	goto		sendXcommand
picMode
	global		picMode
	movlw		0xF9				;PIC mode
	goto		sendXcommand
fpu_break
	global		fpu_break
	movlw		0xFB				;debug breakpoint
	goto		sendXcommand
fpu_traceOff
	global		fpu_traceOff
	movlw		0xFC				;turn debug tracing off
	goto		traceCommand
fpu_traceOn
	global		fpu_traceOn
	movlw		0xFD				;turn debug tracing off
traceCommand:
	call		sendXcommand
	goto		fpu_traceDelay		;delay for trace on/off string	
fpu_traceStr
	global		fpu_traceStr
	movlw		0xFE				;display debug string
	goto		sendXcommand		;(must be followed by string write)
fpu_checksum
	global		fpu_checksum
	clrf		regB				;clear B register
	movlw		0xFF				;calculate checksum and return in reg0
	goto		sendXcommand
fpu_version
	global		fpu_version
	movlw		0xFF				;calculate checksum and return in reg0
	goto		sendCommand			;(must be followed by string read)

;-------------------- load routines -------------------------------------------

load_floatByte
	global		load_floatByte		;write signed byte to reg0 and convert to float
	movlw		0xF4
	goto		loadCommand			;(must be followed by 1 fpu_sendByte call)	
load_floatUbyte
	global		load_floatUbyte		;write unsigned byte to reg0 and convert to float
	movlw		0xF5
	goto		loadCommand			;(must be followed by 1 fpu_sendByte call)	
load_floatWord
	global		load_floatWord		;write signed word to reg0 and convert to float
	movlw		0xF6
	goto		loadCommand			;(must be followed by 2 fpu_sendByte calls)	
load_floatUword
	global		load_floatUword		;write unsigned word to reg0 and convert to float
	movlw		0xF7
	;(falls through to loadCommand) ;(must be followed by 2 fpu_sendByte calls)	
loadCommand
	clrf		regB				;set regB = 0
	goto		sendCommand			;send command

load_zero
	global		load_zero
	movlw		0xF0				;load zero
	goto		loadXcommand
load_one
	global		load_one
	movlw		0xF1				;load one
	goto		loadXcommand
load_e
	global		load_e
	movlw		0xF2				;load e (2.7182817)
	goto		loadXcommand
load_pi
	global		load_pi
	movlw		0xF3				;load pi (3.1415927)
	goto		loadXcommand		

load_longByte
	global		load_longByte		;write signed byte to reg0 and convert to long
	movlw		0xF4
	goto		loadXcommand		;(must be followed by 1 fpu_sendByte call)	
load_longUbyte
	global		load_longUbyte		;write unsigned byte to reg0 and convert to long
	movlw		0xF5
	goto		loadXcommand		;(must be followed by 1 fpu_sendByte call)	
load_longWord
	global		load_longWord		;write signed word to reg0 and convert to long
	movlw		0xF6
	goto		loadXcommand		;(must be followed by 2 fpu_sendByte calls)	
load_longUword
	global		load_longUword		;write unsigned word to reg0 and convert to long
	movlw		0xF7
	;(fall through to loadXcommand)	;(must be followed by 2 fpu_sendByte calls)	
loadXcommand
	clrf		regB				;set regB = 0
	goto		sendXcommand		;send command

get_long
	call		fpu_readDelay		;delay before read
	call		fpu_readByte		;get high word of 32-bit value
	movwf		dataByte+3
	call		fpu_readByte
	movwf		dataByte+2
	call		fpu_readByte
	movwf		dataByte+1
	goto		get_byte2
get_byte
	call		fpu_readDelay		;delay before read
get_byte2	
	call		fpu_readByte		;get 8-bit byte and store
	movwf		dataByte
	return

;-------------------- support routines ----------------------------------------

sendCommandB
	movwf		opcode				;save the opcode

	movf		regB, w				;select B register
	iorlw		0x10
	call		sendCommand

	movf		opcode, w			;check for compare command
	sublw		0xF3
	btfss		STATUS, Z
	goto		sendXcommand2		;no, then double byte command
	movf		opcode, w
	goto		sendCommand
				
sendXcommand
	movwf		opcode				;save the opcode

sendXcommand2
	movlw		0xFE				;send XOP prefix
	call		sendCommand
	movf		opcode, w			;send the opcode
	goto		fpu_sendByte		; and exit

sendOperator
	iorwf		regB, w				;add value of regB to opcode
	;(fall through to sendCommand)

	global		sendCommand
sendCommand
	movwf		dataByte			;store the opcode
	movf		lastRegA, w			;check if regA has changed
	subwf		regA, w
	btfss		STATUS, Z
	goto		sendCommand2

	movf		dataByte, w			;no, then send the opcode
	goto		fpu_sendOpcode		; and exit

sendCommand2
	movf		regA, w				;yes, then select A register
	movwf		lastRegA			;update last value of regA
	call		fpu_sendOpcode
		
	movf		dataByte, w			;send the original opcode
	goto		fpu_sendOpcode		; and exit

	end
